@@ -15,6 +15,7 @@ class PairingHandler;
 class NetworkPacket;
 class LinkProvider;
 class Device;
+class QSslCertificate;
 
 class DeviceLink
     : public QObject
@@ -25,7 +26,7 @@ public:
     enum PairStatus { NotPaired, Paired };
 
     DeviceLink(const QString& deviceId, LinkProvider* parent);
-    virtual ~DeviceLink() = default;
+    ~DeviceLink() override = default;
 
     virtual QString name() = 0;
 
@@ -44,6 +45,8 @@ public:
     //The daemon will periodically destroy unpaired links if this returns false
     virtual bool linkShouldBeKeptAlive() { return false; }
 
+    virtual QSslCertificate certificate() const = 0;
+
 Q_SIGNALS:
     void pairingRequest(PairingHandler* handler);
     void pairingRequestExpired(PairingHandler* handler);
