@@ -1,33 +1,22 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "daemon.h"
 
-#include <QDBusConnection>
+#include <QDBusMetaType>
 #include <QNetworkAccessManager>
 #include <QDebug>
 #include <QPointer>
+#include <QProcess>
 
 #include "core_debug.h"
 #include "kdeconnectconfig.h"
-#include "networkpackage.h"
+#include "networkpacket.h"
+#include "dbushelper.h"
+#include "notificationserverinfo.h"
 
 #ifdef KDECONNECT_BLUETOOTH
     #include "backends/bluetooth/bluetoothlinkprovider.h"
@@ -39,6 +28,9 @@
 #include "backends/devicelink.h"
 #include "backends/linkprovider.h"
 
+//In older Qt released, qAsConst isnt available
+#include "qtcompat_p.h"
+
 static Daemon* s_instance = nullptr;
 
 struct DaemonPrivate
@@ -50,6 +42,7 @@ struct DaemonPrivate
     QMap<QString, Device*> m_devices;
 
     QSet<QString> m_discoveryModeAcquisitions;
+    bool m_testMode;
 };
 
 Daemon* Daemon::instance()
@@ -64,20 +57,36 @@ Daemon::Daemon(QObject* parent, bool testMode)
 {
     Q_ASSERT(!s_instance);
     s_instance = this;
-    qCDebug(KDECONNECT_CORE) << "KdeConnect daemon starting";
+    d->m_testMode = testMode;
+
+    // HACK init may call pure virtual functions from this class so it can't be called directly from the ctor
+    QTimer::singleShot(0, this, &Daemon::init);
+}
+
+void Daemon::init()
+{
+    qCDebug(KDECONNECT_CORE) << "Daemon starting";
 
     //Load backends
-    if (testMode)
+    if (d->m_testMode)
         d->m_linkProviders.insert(new LoopbackLinkProvider());
     else {
         d->m_linkProviders.insert(new LanLinkProvider());
         #ifdef KDECONNECT_BLUETOOTH
             d->m_linkProviders.insert(new BluetoothLinkProvider());
         #endif
+        #ifdef KDECONNECT_LOOPBACK
+            d->m_linkProviders.insert(new LoopbackLinkProvider());
+        #endif
     }
 
-    //Read remebered paired devices
-    const QStringList& list = KdeConnectConfig::instance()->trustedDevices();
+    //Register on DBus
+    qDBusRegisterMetaType< QMap<QString,QString> >();
+    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.kdeconnect"));
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/modules/kdeconnect"), this, QDBusConnection::ExportScriptableContents);
+
+    //Read remembered paired devices
+    const QStringList& list = KdeConnectConfig::instance().trustedDevices();
     for (const QString& id : list) {
         addDevice(new Device(this, id));
     }
@@ -89,11 +98,9 @@ Daemon::Daemon(QObject* parent, bool testMode)
         a->onStart();
     }
 
-    //Register on DBus
-    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.kdeconnect"));
-    QDBusConnection::sessionBus().registerObject(QStringLiteral("/modules/kdeconnect"), this, QDBusConnection::ExportScriptableContents);
+    NotificationServerInfo::instance().init();
 
-    qCDebug(KDECONNECT_CORE) << "KdeConnect daemon started";
+    qCDebug(KDECONNECT_CORE) << "Daemon started";
 }
 
 void Daemon::acquireDiscoveryMode(const QString& key)
@@ -123,11 +130,13 @@ void Daemon::removeDevice(Device* device)
     d->m_devices.remove(device->id());
     device->deleteLater();
     Q_EMIT deviceRemoved(device->id());
+    Q_EMIT deviceListChanged();
 }
 
 void Daemon::cleanDevices()
 {
-    for (Device* device : qAsConst(d->m_devices)) {
+    const auto devs = d->m_devices;
+    for (Device* device : devs) {
         if (device->isTrusted()) {
             continue;
         }
@@ -141,20 +150,25 @@ void Daemon::cleanDevices()
 
 void Daemon::forceOnNetworkChange()
 {
-    qCDebug(KDECONNECT_CORE) << "Sending onNetworkChange to " << d->m_linkProviders.size() << " LinkProviders";
+    qCDebug(KDECONNECT_CORE) << "Sending onNetworkChange to" << d->m_linkProviders.size() << "LinkProviders";
     for (LinkProvider* a : qAsConst(d->m_linkProviders)) {
         a->onNetworkChange();
     }
 }
 
-Device*Daemon::getDevice(const QString& deviceId)
+Device* Daemon::getDevice(const QString& deviceId)
 {
     for (Device* device : qAsConst(d->m_devices)) {
         if (device->id() == deviceId) {
             return device;
         }
     }
-    return Q_NULLPTR;
+    return nullptr;
+}
+
+const QSet<LinkProvider*>& Daemon::getLinkProviders() const
+{
+    return d->m_linkProviders;
 }
 
 QStringList Daemon::devices(bool onlyReachable, bool onlyTrusted) const
@@ -168,23 +182,35 @@ QStringList Daemon::devices(bool onlyReachable, bool onlyTrusted) const
     return ret;
 }
 
-void Daemon::onNewDeviceLink(const NetworkPackage& identityPackage, DeviceLink* dl)
+QMap<QString, QString> Daemon::deviceNames(bool onlyReachable, bool onlyTrusted) const
 {
-    const QString& id = identityPackage.get<QString>(QStringLiteral("deviceId"));
+    QMap<QString, QString> ret;
+    for (Device* device : qAsConst(d->m_devices)) {
+        if (onlyReachable && !device->isReachable()) continue;
+        if (onlyTrusted && !device->isTrusted()) continue;
+        ret[device->id()] = device->name();
+    }
+    return ret;
+}
+
+void Daemon::onNewDeviceLink(const NetworkPacket& identityPacket, DeviceLink* dl)
+{
+    const QString& id = identityPacket.get<QString>(QStringLiteral("deviceId"));
 
     //qCDebug(KDECONNECT_CORE) << "Device discovered" << id << "via" << dl->provider()->name();
 
     if (d->m_devices.contains(id)) {
-        qCDebug(KDECONNECT_CORE) << "It is a known device" << identityPackage.get<QString>(QStringLiteral("deviceName"));
+        qCDebug(KDECONNECT_CORE) << "It is a known device" << identityPacket.get<QString>(QStringLiteral("deviceName"));
         Device* device = d->m_devices[id];
         bool wasReachable = device->isReachable();
-        device->addLink(identityPackage, dl);
+        device->addLink(identityPacket, dl);
         if (!wasReachable) {
             Q_EMIT deviceVisibilityChanged(id, true);
+            Q_EMIT deviceListChanged();
         }
     } else {
-        qCDebug(KDECONNECT_CORE) << "It is a new device" << identityPackage.get<QString>(QStringLiteral("deviceName"));
-        Device* device = new Device(this, identityPackage, dl);
+        qCDebug(KDECONNECT_CORE) << "It is a new device" << identityPacket.get<QString>(QStringLiteral("deviceName"));
+        Device* device = new Device(this, identityPacket, dl);
 
         //we discard the connections that we created but it's not paired.
         if (!isDiscoveringDevices() && !device->isTrusted() && !dl->linkShouldBeKeptAlive()) {
@@ -206,21 +232,41 @@ void Daemon::onDeviceStatusChanged()
         removeDevice(device);
     } else {
         Q_EMIT deviceVisibilityChanged(device->id(), device->isReachable());
+        Q_EMIT deviceListChanged();
     }
 
 }
 
 void Daemon::setAnnouncedName(const QString& name)
 {
-    qCDebug(KDECONNECT_CORE()) << "Announcing name";
-    KdeConnectConfig::instance()->setName(name);
+    qCDebug(KDECONNECT_CORE) << "Announcing name";
+    KdeConnectConfig::instance().setName(name);
     forceOnNetworkChange();
     Q_EMIT announcedNameChanged(name);
 }
 
+void Daemon::setCustomDevices(const QStringList& addresses)
+{
+    auto& config = KdeConnectConfig::instance();
+
+    auto customDevices = config.customDevices();
+    if (customDevices != addresses) {
+        qCDebug(KDECONNECT_CORE) << "Changed list of custom device addresses:" << addresses;
+        config.setCustomDevices(addresses);
+        Q_EMIT customDevicesChanged(addresses);
+
+        forceOnNetworkChange();
+    }
+}
+
+QStringList Daemon::customDevices() const
+{
+    return KdeConnectConfig::instance().customDevices();
+}
+
 QString Daemon::announcedName()
 {
-    return KdeConnectConfig::instance()->name();
+    return KdeConnectConfig::instance().name();
 }
 
 QNetworkAccessManager* Daemon::networkAccessManager()
@@ -264,12 +310,13 @@ void Daemon::addDevice(Device* device)
     d->m_devices[id] = device;
 
     Q_EMIT deviceAdded(id);
+    Q_EMIT deviceListChanged();
 }
 
 QStringList Daemon::pairingRequests() const
 {
     QStringList ret;
-    for(Device* dev: d->m_devices) {
+    for(Device* dev: qAsConst(d->m_devices)) {
         if (dev->hasPairingRequests())
             ret += dev->id();
     }
@@ -283,5 +330,5 @@ Daemon::~Daemon()
 
 QString Daemon::selfId() const
 {
-    return KdeConnectConfig::instance()->deviceId();
+    return KdeConnectConfig::instance().deviceId();
 }
