@@ -10,6 +10,7 @@
 #include <QQmlContext>
 #include <QDBusPendingCall>
 #include <QDBusPendingReply>
+#include <QGuiApplication>
 
 #include "objectfactory.h"
 #include "responsewaiter.h"
@@ -21,7 +22,12 @@
 #include <remotesinksmodel.h>
 #include <pluginmodel.h>
 #include "core/kdeconnectpluginconfig.h"
+#include "openconfig.h"
 #include "interfaces/commandsmodel.h"
+#include "pointerlocker.h"
+#if WITH_WAYLAND == 1
+#include "pointerlockerwayland.h"
+#endif
 
 QObject* createDBusResponse()
 {
@@ -63,6 +69,22 @@ void KdeConnectDeclarativePlugin::registerTypes(const char* uri)
             return new DaemonDbusInterface;
         }
     );
+    qmlRegisterSingletonType<AbstractPointerLocker>("org.kde.kdeconnect", 1, 0, "PointerLocker", [] (QQmlEngine *, QJSEngine *) -> QObject * {
+        AbstractPointerLocker *ret;
+#if WITH_WAYLAND == 1
+        if (qGuiApp->platformName() == QLatin1String("wayland"))
+            ret = new PointerLockerWayland;
+        else
+#endif
+            ret = new PointerLockerQt;
+        return ret;
+    });
+
+    qmlRegisterSingletonType<OpenConfig>(uri, 1, 0, "OpenConfig",
+        [](QQmlEngine*, QJSEngine*) -> QObject* {
+            return new OpenConfig;
+        }
+    );
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
     qmlRegisterAnonymousType<QAbstractItemModel>(uri, 1);
@@ -84,6 +106,7 @@ void KdeConnectDeclarativePlugin::registerTypes(const char* uri)
     registerFactory<ShareDbusInterface>(uri, "ShareDbusInterfaceFactory");
     registerFactory<RemoteSystemVolumeDbusInterface>(uri, "RemoteSystemVolumeDbusInterfaceFactory");
     registerFactory<BigscreenDbusInterface>(uri, "BigscreenDbusInterfaceFactory");
+    registerFactory<VirtualmonitorDbusInterface>(uri, "VirtualmonitorDbusInterfaceFactory");
 }
 
 void KdeConnectDeclarativePlugin::initializeEngine(QQmlEngine* engine, const char* uri)
