@@ -101,12 +101,15 @@ void NotificationsPlugin::addNotification(Notification* noti)
     });
 
     connect(noti, &Notification::actionTriggered, this, &NotificationsPlugin::sendAction);
+    connect(noti, &Notification::replied, this, [this, noti](const QString& message){
+        sendReply(noti->replyId(), message);
+    });
 
     const QString& publicId = newId();
     m_notifications[publicId] = noti;
     m_internalIdToPublicId[internalId] = publicId;
 
-    DBusHelper::sessionBus().registerObject(device()->dbusPath() + QStringLiteral("/notifications/") + publicId, noti, QDBusConnection::ExportScriptableContents);
+    QDBusConnection::sessionBus().registerObject(device()->dbusPath() + QStringLiteral("/notifications/") + publicId, noti, QDBusConnection::ExportScriptableContents);
     Q_EMIT notificationPosted(publicId);
 }
 
